@@ -5,7 +5,7 @@
 
     Global creation environment.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -16,6 +16,7 @@ import types
 import codecs
 import imghdr
 import string
+import unicodedata
 import cPickle as pickle
 from os import path
 from glob import glob
@@ -63,7 +64,7 @@ default_settings = {
 
 # This is increased every time an environment attribute is added
 # or changed to properly invalidate pickle files.
-ENV_VERSION = 38
+ENV_VERSION = 39
 
 
 default_substitutions = set([
@@ -294,6 +295,8 @@ class BuildEnvironment:
                                     # contains all built docnames
         self.dependencies = {}      # docname -> set of dependent file
                                     # names, relative to documentation root
+        self.reread_always = set()  # docnames to re-read unconditionally on
+                                    # next build
 
         # File metadata
         self.metadata = {}          # docname -> dict of metadata items
@@ -343,6 +346,7 @@ class BuildEnvironment:
         """Remove all traces of a source file in the inventory."""
         if docname in self.all_docs:
             self.all_docs.pop(docname, None)
+            self.reread_always.discard(docname)
             self.metadata.pop(docname, None)
             self.dependencies.pop(docname, None)
             self.titles.pop(docname, None)
@@ -425,6 +429,10 @@ class BuildEnvironment:
                                                  '.doctree')):
                     changed.add(docname)
                     continue
+                # check the "reread always" list
+                if docname in self.reread_always:
+                    changed.add(docname)
+                    continue
                 # check the mtime of the document
                 mtime = self.all_docs[docname]
                 newmtime = path.getmtime(self.doc2path(docname))
@@ -490,7 +498,8 @@ class BuildEnvironment:
         # if files were added or removed, all documents with globbed toctrees
         # must be reread
         if added or removed:
-            changed.update(self.glob_toctrees)
+            # ... but not those that already were removed
+            changed.update(self.glob_toctrees & self.found_docs)
 
         msg += '%s added, %s changed, %s removed' % (len(added), len(changed),
                                                      len(removed))
@@ -505,8 +514,7 @@ class BuildEnvironment:
                 self.clear_doc(docname)
 
             # read all new and changed files
-            to_read = added | changed
-            for docname in sorted(to_read):
+            for docname in sorted(added | changed):
                 yield docname
                 self.read_doc(docname, app=app)
 
@@ -623,6 +631,11 @@ class BuildEnvironment:
         codecs.register_error('sphinx', self.warn_and_replace)
 
         class SphinxSourceClass(FileInput):
+            def __init__(self_, *args, **kwds):
+                # don't call sys.exit() on IOErrors
+                kwds['handle_io_errors'] = False
+                FileInput.__init__(self_, *args, **kwds)
+
             def decode(self_, data):
                 return data.decode(self_.encoding, 'sphinx')
 
@@ -729,6 +742,9 @@ class BuildEnvironment:
     def note_dependency(self, filename):
         self.dependencies.setdefault(self.docname, set()).add(filename)
 
+    def note_reread(self):
+        self.reread_always.add(self.docname)
+
     def note_versionchange(self, type, version, node, lineno):
         self.versionchanges.setdefault(version, []).append(
             (type, self.temp_data['docname'], lineno,
@@ -1230,11 +1246,12 @@ class BuildEnvironment:
                         self.warn(docname,
                                   'toctree contains reference to document '
                                   '%r that doesn\'t have a title: no link '
-                                  'will be generated' % ref)
+                                  'will be generated' % ref, toctreenode.line)
                 except KeyError:
                     # this is raised if the included file does not exist
                     self.warn(docname, 'toctree contains reference to '
-                              'nonexisting document %r' % ref)
+                              'nonexisting document %r' % ref,
+                              toctreenode.line)
                 else:
                     # if titles_only is given, only keep the main title and
                     # sub-toctrees
@@ -1298,9 +1315,10 @@ class BuildEnvironment:
             target = node['reftarget']
             refdoc = node.get('refdoc', fromdocname)
             warned = False
+            domain = None
 
             try:
-                if node.has_key('refdomain') and node['refdomain']:
+                if 'refdomain' in node and node['refdomain']:
                     # let the domain try to resolve the reference
                     try:
                         domain = self.domains[node['refdomain']]
@@ -1342,12 +1360,17 @@ class BuildEnvironment:
                     newnode = builder.app.emit_firstresult(
                         'missing-reference', self, node, contnode)
                     # still not found? warn if in nit-picky mode
-                    if newnode is None and not warned and self.config.nitpicky:
-                        self.warn(refdoc,
-                            'reference target not found: %stype %s, target %s'
-                            % (node.get('refdomain') and
-                               'domain %s, ' % node['refdomain'] or '',
-                               typ, target))
+                    if newnode is None and not warned and \
+                       (self.config.nitpicky or node.get('refwarn')):
+                        if domain and typ in domain.dangling_warnings:
+                            msg = domain.dangling_warnings[typ]
+                        elif node.get('refdomain') != 'std':
+                            msg = '%s:%s reference target not found: ' \
+                                  '%%(target)s' % (node['refdomain'], typ)
+                        else:
+                            msg = '%s reference target not found: ' \
+                                  '%%(target)s' % typ
+                        self.warn(refdoc, msg % {'target': target}, node.line)
             except NoUri:
                 newnode = contnode
             node.replace_self(newnode or contnode)
@@ -1363,7 +1386,9 @@ class BuildEnvironment:
                 if ret:
                     node.replace_self(node.children)
                 else:
-                    node.replace_self([])
+                    # replacing by [] would result in an "Losing ids" exception
+                    # if there is a target node before the only node
+                    node.replace_self(nodes.comment())
 
         # allow custom references to be resolved
         builder.app.emit('doctree-resolved', doctree, fromdocname)
@@ -1478,7 +1503,7 @@ class BuildEnvironment:
         # sort the index entries; put all symbols at the front, even those
         # following the letters in ASCII, this is where the chr(127) comes from
         def keyfunc(entry, lcletters=string.ascii_lowercase + '_'):
-            lckey = entry[0].lower()
+            lckey = unicodedata.normalize('NFD', entry[0].lower())
             if lckey[0:1] in lcletters:
                 return chr(127) + lckey
             return lckey
@@ -1520,7 +1545,7 @@ class BuildEnvironment:
             # hack: mutating the subitems dicts to a list in the keyfunc
             v[1] = sorted((si, se) for (si, (se, void)) in v[1].iteritems())
             # now calculate the key
-            letter = k[0].upper()
+            letter = unicodedata.normalize('NFD', k[0])[0].upper()
             if letter in letters:
                 return letter
             else:
