@@ -8,7 +8,7 @@
 
     :author: Sebastian Wiesner
     :contact: basti.wiesner@gmx.net
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -110,7 +110,6 @@ class BuildDoc(Command):
             build = self.get_finalized_command('build')
             self.build_dir = os.path.join(build.build_base, 'sphinx')
             self.mkpath(self.build_dir)
-        self.ensure_dirname('build_dir')
         self.doctree_dir = os.path.join(self.build_dir, 'doctrees')
         self.mkpath(self.doctree_dir)
         self.builder_target_dir = os.path.join(self.build_dir, self.builder)
@@ -137,10 +136,7 @@ class BuildDoc(Command):
                      freshenv=self.fresh_env)
 
         try:
-            if self.all_files:
-                app.builder.build_all()
-            else:
-                app.builder.build_update()
+            app.build(force_all=self.all_files)
         except Exception, err:
             from docutils.utils import SystemMessage
             if isinstance(err, SystemMessage):
